# -*- coding: UTF-8 -*-
# minimum_pandas_version = "0.19.2"
# minimum_pyarrow_version = "0.12.0"
try:
    import warnings

    warnings.filterwarnings("ignore")
    import pandas as pd
    import numpy as np
    import pyarrow as pa
    import struct
    import socket
    import time
    import os
    from pandas.api.types import is_datetime64_dtype, is_datetime64tz_dtype
    from pyspark.serializers import ArrowSerializer, _create_batch
    from pyspark.serializers import BatchedSerializer, PickleSerializer
    from pyspark.sql.types import _parse_datatype_json_string
    from pyspark.sql.types import from_arrow_schema, to_arrow_type, TimestampType
    from tempfile import NamedTemporaryFile
    import sys
    import json
    from py4j.java_gateway import JavaGateway
except ImportError:
    exit(1)

basestring = unicode = str
listType = list
tupleType = tuple
if sys.version >= "3":
    long = int
    xrange = range


def local_connect_and_auth(port):
    for res in socket.getaddrinfo("127.0.0.1", port, socket.AF_UNSPEC, socket.SOCK_STREAM):
        af, socktype, proto, _, sa = res
        try:
            sock = socket.socket(af, socktype, proto)
            sock.settimeout(15)
            sock.connect(sa)
            sockfile = sock.makefile("rwb", 65536)
            return (sockfile, sock)
        except socket.error as e:
            sock.close()
            sock = None
    raise Exception("could not open socket")


def connect_serializer(sock_info, serializer):
    (sockfile, sock) = local_connect_and_auth(sock_info)
    sock.settimeout(None)
    return serializer.load_stream(sockfile)


gateway = JavaGateway()
sparkSession = gateway.jvm.com.finebi.spider.cluster.spark.ContextManager.INSTANCE.getSparkSession()
sc = gateway.jvm.com.finebi.spider.cluster.spark.ContextManager.INSTANCE.getSc()
sqlContext = sparkSession.sqlContext()


def collect(jdf):
    sock_info = jdf.collectToPython()
    sockfile = listType(connect_serializer(sock_info, BatchedSerializer(PickleSerializer())))
    columns = [f.name for f in _parse_datatype_json_string(jdf.schema().json()).fields]
    return pd.DataFrame.from_records(sockfile, columns=columns)


collectDF = collect(gateway.getInput())
PY_INPUT = collectDF.sort_index()
PY_OUTPUT = pd.DataFrame()


# PY_OUTPUT = PY_INPUT
{0}


def serialize_to_jvm(data, parallelism, serializer):
    tempFile = NamedTemporaryFile(delete=False)
    try:
        serializer.dump_stream(data, tempFile)
        tempFile.close()
        readRDDFromFile = gateway.jvm.org.apache.spark.api.python.PythonRDD.readRDDFromFile
        return readRDDFromFile(sc, tempFile.name, parallelism)
    finally:
        os.unlink(tempFile.name)


def create_from_pandas_with_arrow(pdf, timezone):
    schema = [basestring(x) for x in pdf.columns]
    arrow_types = [to_arrow_type(TimestampType()) if is_datetime64_dtype(t) or is_datetime64tz_dtype(t) else None for t
                   in pdf.dtypes]
    step = -(-len(pdf) // 2)
    pdf_slices = (pdf[start:start + step] for start in xrange(0, len(pdf), step))
    batches = [_create_batch([(c, t) for (_, c), t in zip(pdf_slice.iteritems(), arrow_types)], timezone) for pdf_slice
               in pdf_slices]
    if isinstance(schema, (listType, tupleType)):
        struct = from_arrow_schema(batches[0].schema)
        for i, name in enumerate(schema):
            struct.fields[i].name = name
            struct.names[i] = name
        schema = struct
    jrdd = serialize_to_jvm(batches, len(batches), ArrowSerializer())
    jdf = gateway.jvm.org.apache.spark.sql.api.python.PythonSQLUtils.arrowPayloadToDataFrame(jrdd, schema.json(),
                                                                                             sqlContext)
    return jdf

if not isinstance(PY_OUTPUT, pd.DataFrame):
    exit(3)
if len(PY_OUTPUT.index) > 0:
    tail = PY_OUTPUT.tail(1)
    colTypes = tail.select_dtypes(include=['object'])
    if colTypes.empty:
        out = PY_OUTPUT.append(tail)
    else:
        appendDF = tail.fillna(value=dict([(x, "unknown") for x in listType(colTypes)]))
        out = PY_OUTPUT.append(appendDF)
    output = create_from_pandas_with_arrow(out, None)
    gateway.appendOutput(output)
